-- Image Resource Viewer

-- todo
--   bug applying filter sizes - shows extra image at end of library
--   need to rewrite the LibraryWindow and edit features
--   need to reimplement custom library entries
--   need to reimplement import/export
--   need to strip out UI strings for localization

import "Turbine.UI";
import "Turbine.UI.Lotro";
import "GaranStuff.IRV.Class";
import "GaranStuff.IRV.LibraryWindow";
import "GaranStuff.IRV.DropDownList";
import "GaranStuff.IRV.EditLibraryWindow";
import "GaranStuff.IRV.CategoryMaintWindow";
import "GaranStuff.IRV.ImportExportWindow";
import "GaranStuff.IRV.CheckWindow" -- checks image size
import "GaranStuff.IRV.DebugWindow"
import "GaranStuff.IRV.LibraryMaintenance" -- administrative maintenance stuff
import "GaranStuff.IRV.Table"


local fontColor=Turbine.UI.Color(.9,.9,.1);
local fontName=Turbine.UI.Lotro.Font.TrajanPro14;
adm=false
Categories={};
-- Categories are the user defined categories for Library, used for filtering
SubCategories={};
SubCategoryCaption={};
DefaultsLoaded=false;
UpdatesLoaded=false;
minResourceID=0x41000000
maxResourceID=0x4115c3f5 -- subject to change with each release...
inShowBackgrounds=false

function UnloadPlugin()
	SaveData(false);
	-- remove shell commands
	Turbine.Shell.RemoveCommand(irvWindow.shellCmd)
	Turbine.Shell.WriteLine("IRV "..Plugins["IRV"]:GetVersion().." by Garan unloaded");
end

function SaveData(triggerSaveEvent)
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_Categories", Categories );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_SubCategories", SubCategories );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_SubCategoryCaption", SubCategoryCaption );
	local Settings={};
	local resourceID=tonumber(irvWindow.resourceID:GetText());
	if resourceID==nil then resourceID="" end;
	Settings[1]={"ResourceID",tonumber(resourceID)};
	Settings[2]={"NumberFormat",irvWindow.numberFormat};
	Settings[3]={"Adm",adm}
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_Settings", Settings );

	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_LibraryIcons", LibraryIcons );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_LibraryScenes", LibraryScenes );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_LibraryTiles", LibraryTiles );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_LibraryMaps", LibraryMaps );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_LibraryOther", LibraryOther );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_LibraryInvalid", LibraryInvalid );
--	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_LibraryTerrain", LibraryTerrain );
	Turbine.PluginData.Save( Turbine.DataScope.Account, "IRV_LibraryTexture", LibraryTexture );

end
function RebuildIndex(tmpLibrary)
	-- rebuilds the index for the specified library, allows quick ordered lookups
	local tmpIndex={}
	for k,v in pairs(tmpLibrary) do
		table.insert(tmpIndex,k)
	end
	table.sort(tmpIndex)
	for k,v in ipairs(tmpIndex) do
		tmpLibrary[v][6]=k
	end
	if tmpLibrary==LibraryIcons then
		ResourceIcons=tmpIndex
	elseif tmpLibrary==LibraryScenes then
		ResourceScenes=tmpIndex
	elseif tmpLibrary==LibraryTiles then
		ResourceTiles=tmpIndex
	elseif tmpLibrary==LibraryMaps then
		ResourceMaps=tmpIndex
	elseif tmpLibrary==LibraryOther then
		ResourceOther=tmpIndex
	elseif tmpLibrary==LibraryInvalid then
		ResourceInvalid=tmpIndex
	elseif tmpLibrary==LibraryTerrain then
		ResourceTerrain=tmpIndex
	elseif tmpLibrary==LibraryTexture then
		ResourceTexture=tmpIndex
	end
end
function LoadData()
	Categories=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_Categories");
	SubCategories=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_SubCategories");
	SubCategoryCaption=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_SubCategoryCaption");
	if Categories==nil then Categories={} end;
	if SubCategories==nil then SubCategories={} end;
	if SubCategoryCaption==nil then SubCategoryCaption={} end;

	LibraryIcons=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_LibraryIcons");
	LibraryScenes=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_LibraryScenes");
	LibraryTiles=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_LibraryTiles");
	LibraryMaps=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_LibraryMaps");
	LibraryOther=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_LibraryOther");
	LibraryInvalid=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_LibraryInvalid");
-- too many resources in this file, continue to load from defaults. note, these images are useless to plugin devs anyway and are only included for curiousity sake
--	LibraryTerrain=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_LibraryTerrain");
	LibraryTexture=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_LibraryTexture");
	-- if any library failed to load, load the default
	if LibraryIcons==nil then
		import "GaranStuff.IRV.LibraryIcons"
		if LibraryIcons==nil then
			LibraryIcons={}
		end
	end
	if LibraryScenes==nil then
		import "GaranStuff.IRV.LibraryScenes"
		if LibraryScenes==nil then
			LibraryScenes={}
		end
	end
	if LibraryTiles==nil then
		import "GaranStuff.IRV.LibraryTiles"
		if LibraryTiles==nil then
			LibraryTiles={}
		end
	end
	if LibraryMaps==nil then
		import "GaranStuff.IRV.LibraryMaps"
		if LibraryMaps==nil then
			LibraryMaps={}
		end
	end
	if LibraryOther==nil then
		import "GaranStuff.IRV.LibraryOther"
		if LibraryOther==nil then
			LibraryOther={}
		end
	end
	if LibraryInvalid==nil then
		import "GaranStuff.IRV.LibraryInvalid"
		if LibraryInvalid==nil then
			LibraryInvalid={}
		end
	end
	if LibraryTerrain==nil then
		import "GaranStuff.IRV.LibraryTerrain"
		if LibraryTerrain==nil then
			LibraryTerrain={}
		end
	end
	if LibraryTexture==nil then
		import "GaranStuff.IRV.LibraryTexture"
		if LibraryTexture==nil then
			LibraryTexture={}
		end
	end
	-- now build the ordered indexes
	RebuildIndex(LibraryIcons)

	RebuildIndex(LibraryScenes)

	RebuildIndex(LibraryTiles)

	RebuildIndex(LibraryMaps)

	RebuildIndex(LibraryOther)

	RebuildIndex(LibraryInvalid)

	RebuildIndex(LibraryTerrain)

	RebuildIndex(LibraryTexture)

end
LoadData(); -- we want to have all of the categories and other arrays filled before we try to instantiate the windows since some controls depend on this data (like the category list)
function GetLibraryByType(tmpType)
	-- this returns the library table based on its type code
	local tmpLib=LibraryOther
	if tmpType==2 then
		tmpLib=LibraryMaps
	elseif tmpType==3 then
		tmpLib=LibraryScenes
	elseif tmpType==4 then
		tmpLib=LibraryTexture
	elseif tmpType==1 then
		tmpLib=LibraryIcons
	elseif tmpType==6 then
		tmpLib=LibraryTiles
	elseif tmpType==7 then
		tmpLib=LibraryTerrain
	end
	return tmpLib
end
function GetLibraryType(resourceID)
	local tmpType=0
	local tmpText="Undefined"
	local tmpLib=nil
	local tmpRes=nil
	if LibraryIcons[resourceID]~=nil then
		tmpType=1
		tmpText="Icons"
		tmpLib=LibraryIcons
		tmpRes=ResourceIcons
	elseif LibraryMaps[resourceID]~=nil then
		tmpType=2
		tmpText="Maps"
		tmpLib=LibraryMaps
		tmpRes=ResourceMaps
	elseif LibraryScenes[resourceID]~=nil then
		tmpType=3
		tmpText="Scenes"
		tmpLib=LibraryScenes
		tmpRes=ResourceScenes
	elseif LibraryTexture[resourceID]~=nil then
		tmpType=4
		tmpText="Texture"
		tmpLib=LibraryTexture
		tmpRes=ResourceTexture
	elseif LibraryOther[resourceID]~=nil then
		tmpType=5
		tmpText="Other"
		tmpLib=LibraryOther
		tmpRes=ResourceOther
	elseif LibraryTiles[resourceID]~=nil then
		tmpType=6
		tmpText="Tiles"
		tmpLib=LibraryTiles
		tmpRes=ResourceTiles
	elseif LibraryTerrain[resourceID]~=nil then
		tmpType=7
		tmpText="Terrain"
		tmpLib=LibraryTerrain
		tmpRes=ResourceTerrain
	end
	return tmpType,tmpText,tmpLib,tmpRes
end
function GetLibraryInfo(resourceID)
	local category="";
	local subCategory="";
	local description="Undefined";
	local libIndex=1;
	-- find the resource and return it's Class, Subclass and Description
	-- we have to do a table scan since the table is indexed by Category/Description not resourceID :(

	local tmpType,tmpText,tmpLib=GetLibraryType(resourceID)

	if tmpLib~=nil and tmpLib[resourceID]~=nil then
		local tmpEntry=tmpLib[resourceID]
		x=tmpEntry[2]
		y=tmpEntry[3]
		libIndex=tmpEntry[6]
		description=tmpEntry[5]

--Library[assetID]={type,width,height,subtype,description,resourceIndex}
		local found=false
		-- now look up the Category;
		local catIndex=0;
		local subCatIndex=0;
		for k,v in ipairs(Categories) do
			if tmpEntry[1]==v[1] then
				catIndex=k;
				category=v[2];
				break;
			end
		end
		if catIndex>0 then
			local subCat=SubCategories[tmpEntry[1]]
			if tmpEntry[4]~=nil and subCat~=nil then
				for k,v in ipairs(subCat) do
					if subCat[1]==tmpEntry[4] then
						subCatIndex=k
						subcategory=v[2];
						break
					end
				end
			end
		end
	end
	return category,subcategory,description;
end
-- build the main window
irvWindow = Turbine.UI.Window();
	irvWindow.Loading=true;
	irvWindow:SetWantsUpdates( true );
	irvWindow.ShowGrid=false;
	irvWindow.AutoPrev=false;
	irvWindow.AutoNext=false;
	irvWindow.delayTime=0;
	irvWindow.delayTotal=0;
	irvWindow.displayingWarning=false;
	irvWindow.numberFormat="0x%x";
	irvWindow.mergingUpdate=false;

	local displayWidth, displayHeight = Turbine.UI.Display.GetSize();
	local windowWidth, windowHeight = Turbine.UI.Display.GetSize();
	local positionLeft=-1; -- default it to an invalid value so we know if we get a saved value
	local positionTop=-1;
	local Settings=Turbine.PluginData.Load( Turbine.DataScope.Account, "IRV_Settings");
	local lastResourceID;
	if Settings ~=nil then
		local lIndex;
		for lIndex=1,#Settings do
			if Settings[lIndex][1]=="Width" then
				if Settings[lIndex][2]<windowWidth then
					windowWidth=Settings[lIndex][2];
				end
			end
			if Settings[lIndex][1]=="Height" then
				if Settings[lIndex][2]<windowHeight then
					windowHeight=Settings[lIndex][2];
				end
			end
			if Settings[lIndex][1]=="Left" then
				if Settings[lIndex][2]<positionLeft then
					positionLeft=Settings[lIndex][2];
				end
			end
			if Settings[lIndex][1]=="Top" then
				if Settings[lIndex][2]<positionTop then
					positionTop=Settings[lIndex][2];
				end
			end
			if Settings[lIndex][1]=="NumberFormat" then
				irvWindow.numberFormat=Settings[lIndex][2];
			end
			if Settings[lIndex][1]=="ResourceID" then
				lastResourceID=tonumber(Settings[lIndex][2]);
			end
			if Settings[lIndex][1]=="Adm" then
				adm=Settings[lIndex][2];
			end
		end
	end
	if lastResourceID==nil then lastResourceID=0x41000000 end;
	local lastType=GetLibraryType(lastResourceID)

	irvWindow:SetSize(windowWidth,windowHeight);
	if positionLeft==-1 then positionLeft=( displayWidth - windowWidth ) / 2 end;
	if positionTop==-1 then positionTop=( displayWidth - windowWidth ) / 2 end;
	irvWindow:SetPosition( positionLeft, positionTop);
	irvWindow:SetBackColor(Turbine.UI.Color(0,.01,.05));
	irvWindow:SetOpacity(.95);
	irvWindow:SetText("Image Resource Viewer");

	irvWindow.TypeCaption=Turbine.UI.Label()
	irvWindow.TypeCaption:SetParent(irvWindow)
	irvWindow.TypeCaption:SetSize(100,20)
	irvWindow.TypeCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	irvWindow.TypeCaption:SetPosition(0,0)
	irvWindow.TypeCaption:SetText("Image Type:")
	irvWindow.TypeList=DropDownList()
	irvWindow.TypeList:SetParent(irvWindow)
	irvWindow.TypeList:SetSize(120,20)
	irvWindow.TypeList:SetPosition(irvWindow.TypeCaption:GetLeft()+irvWindow.TypeCaption:GetWidth(),irvWindow.TypeCaption:GetTop())
	irvWindow.TypeList:SetBackColor(Turbine.UI.Color.Black);
	irvWindow.TypeList:SetCurrentBackColor(Turbine.UI.Color.Black);
	irvWindow.TypeList:SetBorderColor(Turbine.UI.Color.Gold);
	irvWindow.TypeList:SetTextColor(Turbine.UI.Color.White);

	irvWindow.TypeList:AddItem("Icons (32x32)",1)
	irvWindow.TypeList:AddItem("Maps",2)
	irvWindow.TypeList:AddItem("Scenes",3)
	irvWindow.TypeList:AddItem("Textures",4)
	irvWindow.TypeList:AddItem("Miscellaneous",5)
	irvWindow.TypeList:AddItem("Compass Tiles",6)
	irvWindow.TypeList:AddItem("Elevations",7)
	irvWindow.TypeList:SetSelectedIndex(lastType)
	irvWindow.TypeList.SelectedIndexChanged=function()
		--
		local tmpResource=tonumber(irvWindow.resourceID:GetText())
		local tmpType,b,c,d=GetLibraryType(tmpResource)
		if tmpType~=irvWindow.TypeList:GetValue() then
			irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,GetNextResource(tmpResource)));
		end
		ShowBackgrounds(false);
	end

	irvWindow.TargetCaption=Turbine.UI.Label()
	irvWindow.TargetCaption:SetParent(irvWindow)
	irvWindow.TargetCaption:SetSize(100,20)
	irvWindow.TargetCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	irvWindow.TargetCaption:SetPosition(irvWindow.TypeCaption:GetLeft(),irvWindow.TypeCaption:GetTop()+irvWindow.TypeCaption:GetHeight())
	irvWindow.TargetCaption:SetText("Target Type:")
	irvWindow.TargetCaption:SetVisible(adm)
	irvWindow.TargetList=DropDownList()
	irvWindow.TargetList:SetParent(irvWindow)
	irvWindow.TargetList:SetSize(120,20)
	irvWindow.TargetList:SetPosition(irvWindow.TargetCaption:GetLeft()+irvWindow.TargetCaption:GetWidth(),irvWindow.TargetCaption:GetTop())
	irvWindow.TargetList:SetBackColor(Turbine.UI.Color.Black);
	irvWindow.TargetList:SetCurrentBackColor(Turbine.UI.Color.Black);
	irvWindow.TargetList:SetBorderColor(Turbine.UI.Color.Gold);
	irvWindow.TargetList:SetTextColor(Turbine.UI.Color.White);
	irvWindow.TargetList:SetVisible(adm)

	irvWindow.TargetList:AddItem("Icons (32x32)",1)
	irvWindow.TargetList:AddItem("Maps",2)
	irvWindow.TargetList:AddItem("Scenes",3)
	irvWindow.TargetList:AddItem("Textures",4)
	irvWindow.TargetList:AddItem("Miscellaneous",5)
	irvWindow.TargetList:AddItem("Compass Tiles",6)
	irvWindow.TargetList:AddItem("Elevations",7)
	irvWindow.TargetList:SetSelectedIndex(lastType)
	irvWindow.TargetList.SelectedIndexChanged=function()
		--
	end

	irvWindow.resourceCaption=Turbine.UI.Label();
	irvWindow.resourceCaption:SetParent(irvWindow);
	irvWindow.resourceCaption:SetSize(90,18);
	irvWindow.resourceCaption:SetPosition(irvWindow.TypeList:GetLeft()+irvWindow.TypeList:GetWidth()+10,irvWindow.TypeList:GetTop());
	irvWindow.resourceCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	irvWindow.resourceCaption:SetText("Resource ID:");

	irvWindow.resourceID=Turbine.UI.Lotro.TextBox();
	irvWindow.resourceID:SetParent(irvWindow);
	irvWindow.resourceID:SetPosition(irvWindow.resourceCaption:GetLeft()+irvWindow.resourceCaption:GetWidth()+5,irvWindow.resourceCaption:GetTop()-2)
	irvWindow.resourceID:SetSize(100,18);
	irvWindow.resourceID:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	irvWindow.resourceID:SetBackColor(Turbine.UI.Color(.1,.1,.5));
	irvWindow.resourceID:SetForeColor(Turbine.UI.Color.White);
	irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,lastResourceID));

	irvWindow.resourceDescription=Turbine.UI.Label();
	irvWindow.resourceDescription:SetParent(irvWindow);
	irvWindow.resourceDescription:SetPosition(irvWindow.TargetList:GetLeft()+irvWindow.TargetList:GetWidth()+5,24);
	irvWindow.resourceDescription:SetSize(irvWindow:GetWidth()-irvWindow.resourceDescription:GetLeft(),18);
	irvWindow.resourceDescription:SetFont(Turbine.UI.Lotro.Font.Verdana14);
	irvWindow.resourceDescription:SetText("");

	irvWindow.imageSize=Turbine.UI.Label();
	irvWindow.imageSize:SetParent(irvWindow);
	irvWindow.imageSize:SetSize(70,18);
	irvWindow.imageSize:SetPosition(irvWindow.resourceID:GetLeft()+irvWindow.resourceID:GetWidth()+5,irvWindow.resourceCaption:GetTop());
	irvWindow.imageSize:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);

	irvWindow.imageSolo=Turbine.UI.Label();
	irvWindow.imageSolo:SetParent(irvWindow);
	irvWindow.imageSolo:SetVisible(false)
	irvWindow.imageSolo:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	irvWindow.imageSolo.MouseClick=function(sender,args)
		if args.Button==2 then
			irvWindow.imageMenu:GetItems():Get(1):SetText("");
			irvWindow.imageMenu:GetItems():Get(2):SetText("");
			irvWindow.imageMenu:GetItems():Get(3):SetText("");
			irvWindow.imageMenu:GetItems():Get(4).Click=function()
				editLibWindow:SetResource(tonumber(irvWindow.resourceID:GetText()),nil,irvWindow.imageWidth,irvWindow.imageHeight);
				editLibWindow:SetVisible(true);
			end
			irvWindow.imageGrid.clickRow=nil
			irvWindow.imageMenu:ShowMenu();
		end
	end
	irvWindow.imageGrid={};

	local width,height=120,120;
	local gridSpace=6;
	irvWindow.GridRows=math.floor(748/(height+gridSpace));
	irvWindow.GridCols=math.floor(1024/(width+gridSpace));
	local gridLeft=(windowWidth-width*irvWindow.GridCols-gridSpace*(irvWindow.GridCols-1))/2
	local gridTop=(windowHeight-20-height*(irvWindow.GridRows)-gridSpace*(irvWindow.GridRows-1))/2

	for iRow=1, irvWindow.GridRows do
		irvWindow.imageGrid[iRow]={}
		for iCol=1, irvWindow.GridCols do
			irvWindow.imageGrid[iRow][iCol]=Turbine.UI.Label();
			irvWindow.imageGrid[iRow][iCol].imageX=0;
			irvWindow.imageGrid[iRow][iCol].imageY=0;
			irvWindow.imageGrid[iRow][iCol].selected=false;
			irvWindow.imageGrid[iRow][iCol]:SetParent(irvWindow);
			irvWindow.imageGrid[iRow][iCol]:SetSize(width,height);
			irvWindow.imageGrid[iRow][iCol]:SetPosition(gridLeft+(iCol-1)*(width+gridSpace),gridTop+(iRow-1)*(height+gridSpace));
			irvWindow.imageGrid[iRow][iCol]:SetBackColor(Turbine.UI.Color(.1,.1,.1));
			irvWindow.imageGrid[iRow][iCol].MouseClick=function(sender,args)
				if args.Button==1 then
					if irvWindow:IsShiftKeyDown() then
						if irvWindow.imageGrid.selectStartRow==nil then
							irvWindow.imageGrid.selectStartRow=iRow
							irvWindow.imageGrid.selectStartCol=iCol
						else
							local endRow,endCol
							if iRow<irvWindow.imageGrid.selectStartRow then
								endRow=irvWindow.imageGrid.selectStartRow
								irvWindow.imageGrid.selectStartRow=iRow
							else
								endRow=iRow
							end
							if iCol<irvWindow.imageGrid.selectStartCol then
								endCol=irvWindow.imageGrid.selectStartCol
								irvWindow.imageGrid.selectStartCol=iCol
							else
								endCol=iCol
							end
							for tmpRow=irvWindow.imageGrid.selectStartRow,endRow do
								for tmpCol=irvWindow.imageGrid.selectStartCol,endCol do
									irvWindow.imageGrid[tmpRow][tmpCol].selected=true
									irvWindow.imageGrid[tmpRow][tmpCol]:SetBackColor(Turbine.UI.Color.DarkBlue)
								end
							end
						end
					elseif irvWindow.IsControlKeyDown() then
						irvWindow.imageGrid.selectStartRow=nil
						irvWindow.imageGrid.selectStartCol=nil
						irvWindow.imageGrid[iRow][iCol].selected=not irvWindow.imageGrid[iRow][iCol].selected
						if irvWindow.imageGrid[iRow][iCol].selected then
							irvWindow.imageGrid[iRow][iCol]:SetBackColor(Turbine.UI.Color.DarkBlue)
						else
							irvWindow.imageGrid[iRow][iCol]:SetBackColor(Turbine.UI.Color(.1,.1,.1));
						end
					else
						irvWindow.imageGrid.selectStartRow=nil
						irvWindow.imageGrid.selectStartCol=nil
						irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,tonumber(irvWindow.imageGrid[iRow][iCol].imageID:GetText())))
						irvWindow.ShowGrid=false;
						irvWindow.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleSingle.jpg");
						ShowBackgrounds(false);
					end
				else
					irvWindow.imageGrid[iRow][iCol].Cat, irvWindow.imageGrid[iRow][iCol].SubCat, irvWindow.imageGrid[iRow][iCol].Desc=GetLibraryInfo(irvWindow.imageGrid[iRow][iCol].ResourceID);
					irvWindow.imageMenu:GetItems():Get(1):SetText(tostring(irvWindow.imageGrid[iRow][iCol].Cat));
					irvWindow.imageMenu:GetItems():Get(2):SetText(irvWindow.imageGrid[iRow][iCol].SubCat);
					irvWindow.imageMenu:GetItems():Get(3):SetText(irvWindow.imageGrid[iRow][iCol].Desc);
					irvWindow.imageMenu:GetItems():Get(4).Click=function()
						editLibWindow:SetResource(tonumber(irvWindow.imageGrid[iRow][iCol].imageID:GetText()),nil,irvWindow.imageGrid[iRow][iCol].imageX,irvWindow.imageGrid[iRow][iCol].imageY);
						editLibWindow:SetVisible(true);
					end
					irvWindow.imageGrid.clickRow=iRow
					irvWindow.imageGrid.clickCol=iCol
					irvWindow.imageMenu:ShowMenu();
				end
			end

			irvWindow.imageGrid[iRow][iCol].image=Turbine.UI.Label();
			irvWindow.imageGrid[iRow][iCol].image:SetParent(irvWindow.imageGrid[iRow][iCol])
			irvWindow.imageGrid[iRow][iCol].image:SetSize(32,32);
			irvWindow.imageGrid[iRow][iCol].image:SetPosition((width-32)/2,1);
			irvWindow.imageGrid[iRow][iCol].image:SetBlendMode(Turbine.UI.BlendMode.Overlay);
			irvWindow.imageGrid[iRow][iCol].Cat="";
			irvWindow.imageGrid[iRow][iCol].SubCat="";
			irvWindow.imageGrid[iRow][iCol].Desc="";
			irvWindow.imageGrid[iRow][iCol].ResourceID=0;
			irvWindow.imageGrid[iRow][iCol].image:SetMouseVisible(false)
			irvWindow.imageGrid[iRow][iCol].imageID=Turbine.UI.Label();
			irvWindow.imageGrid[iRow][iCol].imageID:SetParent(irvWindow.imageGrid[iRow][iCol])
			irvWindow.imageGrid[iRow][iCol].imageID:SetSize(width,14);
			irvWindow.imageGrid[iRow][iCol].imageID:SetPosition(0,height-28);
			irvWindow.imageGrid[iRow][iCol].imageID:SetFont(Turbine.UI.Lotro.Font.TrajanPro12);
			irvWindow.imageGrid[iRow][iCol].imageID:SetBackColor(Turbine.UI.Color(.2,.2,.3));
			irvWindow.imageGrid[iRow][iCol].imageID:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft);
			irvWindow.imageGrid[iRow][iCol].imageID:SetMouseVisible(false)

			irvWindow.imageGrid[iRow][iCol].imageSize=Turbine.UI.Label();
			irvWindow.imageGrid[iRow][iCol].imageSize:SetParent(irvWindow.imageGrid[iRow][iCol])
			irvWindow.imageGrid[iRow][iCol].imageSize:SetSize(width,14);
			irvWindow.imageGrid[iRow][iCol].imageSize:SetPosition(0,height-14);
			irvWindow.imageGrid[iRow][iCol].imageSize:SetFont(Turbine.UI.Lotro.Font.TrajanPro12);
			irvWindow.imageGrid[iRow][iCol].imageSize:SetBackColor(Turbine.UI.Color(.2,.2,.3));
			irvWindow.imageGrid[iRow][iCol].imageSize:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft);
			irvWindow.imageGrid[iRow][iCol].imageSize:SetMouseVisible(false)
			irvWindow.imageGrid[iRow][iCol]:SetVisible(false);
		end
	end

	irvWindow.btnFirstIDs=Turbine.UI.Button();
	irvWindow.btnFirstIDs:SetParent(irvWindow)
	irvWindow.btnFirstIDs:SetSize(26,20);
	irvWindow.btnFirstIDs:SetPosition(irvWindow.imageSize:GetLeft()+irvWindow.imageSize:GetWidth()+5,irvWindow.resourceCaption:GetTop());
	irvWindow.btnFirstIDs:SetBackground("GaranStuff/IRV/Resources/FirstButton.jpg")
	irvWindow.btnFirstIDs.Click=function(sender,args)
		irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,GetNextResource(1)));
		ShowBackgrounds(false);
	end

	irvWindow.btnPrevIDs=Turbine.UI.Button();
	irvWindow.btnPrevIDs:SetParent(irvWindow)
	irvWindow.btnPrevIDs:SetSize(26,20);
	irvWindow.btnPrevIDs:SetPosition(irvWindow.btnFirstIDs:GetLeft()+irvWindow.btnFirstIDs:GetWidth()+5,irvWindow.resourceCaption:GetTop());
	irvWindow.btnPrevIDs:SetBackground("GaranStuff/IRV/Resources/BlankButton.jpg")
	irvWindow.btnPrevIDs.Image=Turbine.UI.Label();
	irvWindow.btnPrevIDs.Image:SetParent(irvWindow.btnPrevIDs);
	irvWindow.btnPrevIDs.Image:SetSize(16,16);
	irvWindow.btnPrevIDs.Image:SetPosition((irvWindow.btnPrevIDs:GetWidth()-16)/2,(irvWindow.btnPrevIDs:GetHeight()-16)/2)
	irvWindow.btnPrevIDs.Image:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	irvWindow.btnPrevIDs.Image:SetBackground(0x41007e0f);
	irvWindow.btnPrevIDs.Image:SetMouseVisible(false);
	irvWindow.btnPrevIDs.Click=function(sender,args)
		local tmpResource=GetPrevResource(tonumber(irvWindow.resourceID:GetText()))
		if tmpResource~=nil then
			irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,tmpResource));
		end
		ShowBackgrounds(true);
	end

	irvWindow.btnGoToImage=Turbine.UI.Button();
	irvWindow.btnGoToImage:SetParent(irvWindow)
	irvWindow.btnGoToImage:SetSize(26,20);
	irvWindow.btnGoToImage:SetPosition(irvWindow.btnPrevIDs:GetLeft()+irvWindow.btnPrevIDs:GetWidth()+5,irvWindow.resourceCaption:GetTop());
	irvWindow.btnGoToImage:SetBackground("GaranStuff/IRV/Resources/BlankButton.jpg")
	irvWindow.btnGoToImage.Image=Turbine.UI.Label();
	irvWindow.btnGoToImage.Image:SetParent(irvWindow.btnGoToImage);
	irvWindow.btnGoToImage.Image:SetSize(18,18);
	irvWindow.btnGoToImage.Image:SetPosition((irvWindow.btnGoToImage:GetWidth()-18)/2,(irvWindow.btnGoToImage:GetHeight()-18)/2)
	irvWindow.btnGoToImage.Image:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	irvWindow.btnGoToImage.Image:SetBackground(0x41007dd4);
	irvWindow.btnGoToImage.Image:SetMouseVisible(false);
	irvWindow.btnGoToImage.Click=function(sender,args)
		ShowBackgrounds(false);
	end

	irvWindow.btnNextIDs=Turbine.UI.Button();
	irvWindow.btnNextIDs:SetParent(irvWindow)
	irvWindow.btnNextIDs:SetSize(26,20);
	irvWindow.btnNextIDs:SetPosition(irvWindow.btnGoToImage:GetLeft()+irvWindow.btnGoToImage:GetWidth()+5,irvWindow.resourceCaption:GetTop());
	irvWindow.btnNextIDs:SetBackground("GaranStuff/IRV/Resources/BlankButton.jpg")
	irvWindow.btnNextIDs.Image=Turbine.UI.Label();
	irvWindow.btnNextIDs.Image:SetParent(irvWindow.btnNextIDs);
	irvWindow.btnNextIDs.Image:SetSize(16,16);
	irvWindow.btnNextIDs.Image:SetPosition((irvWindow.btnNextIDs:GetWidth()-16)/2,(irvWindow.btnNextIDs:GetHeight()-16)/2)
	irvWindow.btnNextIDs.Image:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	irvWindow.btnNextIDs.Image:SetBackground(0x41007e12);
	irvWindow.btnNextIDs.Image:SetMouseVisible(false);
	irvWindow.btnNextIDs.Click=function(sender,args)
		local tmpResource
		if irvWindow.ShowGrid then
			tmpResource=tonumber(irvWindow.imageGrid[irvWindow.GridRows][irvWindow.GridCols].ResourceID)
		else
			tmpResource=tonumber(irvWindow.resourceID:GetText())
		end
		irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,GetNextResource(tmpResource)));
		ShowBackgrounds(false);
	end

	irvWindow.btnLastIDs=Turbine.UI.Button();
	irvWindow.btnLastIDs:SetParent(irvWindow)
	irvWindow.btnLastIDs:SetSize(26,20);
	irvWindow.btnLastIDs:SetPosition(irvWindow.btnNextIDs:GetLeft()+irvWindow.btnNextIDs:GetWidth()+5,irvWindow.resourceCaption:GetTop());
	irvWindow.btnLastIDs:SetBackground("GaranStuff/IRV/Resources/LastButton.jpg")
	irvWindow.btnLastIDs.Click=function(sender,args)
		local tmpResource=GetNextResource(0xFFFFFFFF)
		if irvWindow.ShowGrid then
			for i=1,#irvWindow.imageGrid*#irvWindow.imageGrid[1]-1 do
				tmpResource=GetPrevResource(tmpResource)
			end
		end
		irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,tmpResource));
		ShowBackgrounds(false);
	end

	irvWindow.XofY=Turbine.UI.Label()
	irvWindow.XofY:SetParent(irvWindow)
	irvWindow.XofY:SetSize(150,15)
	irvWindow.XofY:SetPosition(irvWindow.btnLastIDs:GetLeft()+irvWindow.btnLastIDs:GetWidth()+5,irvWindow.resourceCaption:GetTop());
	irvWindow.XofY:SetFont(Turbine.UI.Lotro.Font.Verdana12);
	irvWindow.XofY:SetTextAlignment( Turbine.UI.ContentAlignment.TopCenter);
	irvWindow.XofYScroll=Turbine.UI.Lotro.ScrollBar()
	irvWindow.XofYScroll:SetParent(irvWindow);
	irvWindow.XofYScroll:SetSize(irvWindow.XofY:GetWidth(),10);
	irvWindow.XofYScroll:SetPosition(irvWindow.XofY:GetLeft(),irvWindow.XofY:GetTop()+12);
	irvWindow.XofYScroll:SetOrientation(Turbine.UI.Orientation.Horizontal);
	irvWindow.XofYScroll:SetMinimum(1)
	irvWindow.XofYScroll:SetSmallChange(irvWindow.GridRows*irvWindow.GridCols)
	irvWindow.XofYScroll:SetLargeChange(100)
	irvWindow.XofYScroll.ValueChanged=function(sender,args)
		if not inShowBackgrounds then
			-- get library of current resource
			local tmpResource=tonumber(irvWindow.resourceID:GetText())
			local newIndex=irvWindow.XofYScroll:GetValue()
			if tmpResource~=nil then
				local a,b,c,d=GetLibraryType(tmpResource)
				irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,d[newIndex]))
				ShowBackgrounds(false)
			end
		end
	end

	irvWindow.btnGridToggle=Turbine.UI.Button();
	irvWindow.btnGridToggle:SetParent(irvWindow)
	irvWindow.btnGridToggle:SetSize(40,20);
	irvWindow.btnGridToggle:SetPosition(irvWindow.XofY:GetLeft()+irvWindow.XofY:GetWidth()+5,irvWindow.resourceCaption:GetTop());
	irvWindow.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleSingle.jpg");
	irvWindow.btnGridToggle.Click=function(sender,args)
		irvWindow.ShowGrid= not irvWindow.ShowGrid
		if irvWindow.ShowGrid==true then
			irvWindow.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleGrid.jpg");
		else
			irvWindow.btnGridToggle:SetBackground("GaranStuff/IRV/Resources/ToggleSingle.jpg");
		end
		ShowBackgrounds(false);
	end

	irvWindow.btnFormatToggle=Turbine.UI.Button();
	irvWindow.btnFormatToggle:SetParent(irvWindow)
	irvWindow.btnFormatToggle:SetSize(40,20);
	irvWindow.btnFormatToggle:SetPosition(irvWindow.btnGridToggle:GetLeft()+irvWindow.btnGridToggle:GetWidth()+6,irvWindow.resourceCaption:GetTop());
	if irvWindow.numberFormat=="0x%x" then
		irvWindow.btnFormatToggle:SetBackground("GaranStuff/IRV/Resources/ToggleHex.jpg");
	else
		irvWindow.btnFormatToggle:SetBackground("GaranStuff/IRV/Resources/ToggleDec.jpg");
	end
	irvWindow.btnFormatToggle.Click=function(sender,args)
		if irvWindow.numberFormat=="0x%x" then
			irvWindow.numberFormat="%d";
			irvWindow.btnFormatToggle:SetBackground("GaranStuff/IRV/Resources/ToggleDec.jpg");
		else
			irvWindow.numberFormat="0x%x";
			irvWindow.btnFormatToggle:SetBackground("GaranStuff/IRV/Resources/ToggleHex.jpg");
		end
		irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,irvWindow.resourceID:GetText()));
		ShowBackgrounds(false);
	end

	irvWindow.btnSearch=Turbine.UI.Button();
	irvWindow.btnSearch:SetParent(irvWindow)
	irvWindow.btnSearch:SetSize(26,20);
	irvWindow.btnSearch:SetPosition(irvWindow.btnFormatToggle:GetLeft()+irvWindow.btnFormatToggle:GetWidth()+6,irvWindow.resourceCaption:GetTop());
	irvWindow.btnSearch:SetBackground("GaranStuff/IRV/Resources/SearchButton.jpg");
	irvWindow.btnSearch.Click=function(sender,args)
		libWindow:SetVisible(true);
	end
-- hide the search button until custom library entries are reimplemented - not sure whether the existing category/subcategory structures and data will be retained
irvWindow.btnSearch:SetVisible(false)

	irvWindow.FilterShow=Turbine.UI.Lotro.Button()
	irvWindow.FilterShow:SetParent(irvWindow)
	irvWindow.FilterShow:SetSize(100,20)
	irvWindow.FilterShow:SetPosition(irvWindow.btnSearch:GetLeft()+irvWindow.btnSearch:GetWidth()+5,irvWindow.resourceCaption:GetTop())
	irvWindow.FilterShow:SetText("Filter")
	irvWindow.FilterShow.Click=function()
		irvWindow.FilterShow:SetVisible(false)
		irvWindow.FilterPanel:SetVisible(true)
	end
	irvWindow.StretchThumbs=Turbine.UI.Lotro.CheckBox()
	irvWindow.StretchThumbs:SetParent(irvWindow)
	irvWindow.StretchThumbs:SetSize(70,20)
	irvWindow.StretchThumbs:SetPosition(irvWindow.FilterShow:GetLeft()+irvWindow.FilterShow:GetWidth()+5,irvWindow.resourceCaption:GetTop())
	irvWindow.StretchThumbs:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	irvWindow.StretchThumbs:SetText("Stretch")
	irvWindow.StretchThumbs:SetChecked(false)
	irvWindow.StretchThumbs.CheckedChanged=function()
		ShowBackgrounds()
	end

	irvWindow.FilterPanel=Turbine.UI.Control()
	irvWindow.FilterPanel:SetParent(irvWindow)
	irvWindow.FilterPanel:SetSize(300,90)
	irvWindow.FilterPanel:SetBackColor(Turbine.UI.Color.Gold)
	irvWindow.FilterPanel:SetPosition(irvWindow:GetWidth()-irvWindow.FilterPanel:GetWidth(),48)
	irvWindow.FilterPanel:SetVisible(false)
	irvWindow.FilterBack=Turbine.UI.Control()
	irvWindow.FilterBack:SetParent(irvWindow.FilterPanel)
	irvWindow.FilterBack:SetSize(irvWindow.FilterPanel:GetWidth()-2,irvWindow.FilterPanel:GetHeight()-2)
	irvWindow.FilterBack:SetPosition(1,1)
	irvWindow.FilterBack:SetBackColor(Turbine.UI.Color.DarkBlue)
	irvWindow.FilterWidthCaption=Turbine.UI.Label()
	irvWindow.FilterWidthCaption:SetParent(irvWindow.FilterPanel)
	irvWindow.FilterWidthCaption:SetSize(120,20)
	irvWindow.FilterWidthCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	irvWindow.FilterWidthCaption:SetPosition(5,5)
	irvWindow.FilterWidthCaption:SetText("Filter Width:")
	irvWindow.FilterWidth=Turbine.UI.Lotro.TextBox()
	irvWindow.FilterWidth:SetParent(irvWindow.FilterPanel)
	irvWindow.FilterWidth:SetSize(60,20)
	irvWindow.FilterWidth:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	irvWindow.FilterWidth:SetPosition(irvWindow.FilterWidthCaption:GetLeft()+irvWindow.FilterWidthCaption:GetWidth(),irvWindow.FilterWidthCaption:GetTop())
	irvWindow.FilterWidth:SetText("")

	irvWindow.FilterHeightCaption=Turbine.UI.Label()
	irvWindow.FilterHeightCaption:SetParent(irvWindow.FilterPanel)
	irvWindow.FilterHeightCaption:SetSize(120,20)
	irvWindow.FilterHeightCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	irvWindow.FilterHeightCaption:SetPosition(irvWindow.FilterWidthCaption:GetLeft(),irvWindow.FilterWidthCaption:GetTop()+irvWindow.FilterWidthCaption:GetHeight())
	irvWindow.FilterHeightCaption:SetText("Filter Height:")
	irvWindow.FilterHeight=Turbine.UI.Lotro.TextBox()
	irvWindow.FilterHeight:SetParent(irvWindow.FilterPanel)
	irvWindow.FilterHeight:SetSize(60,20)
	irvWindow.FilterHeight:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	irvWindow.FilterHeight:SetPosition(irvWindow.FilterHeightCaption:GetLeft()+irvWindow.FilterHeightCaption:GetWidth(),irvWindow.FilterHeightCaption:GetTop())
	irvWindow.FilterHeight:SetText("")

	irvWindow.FilterClose=Turbine.UI.Lotro.Button()
	irvWindow.FilterClose:SetParent(irvWindow.FilterPanel)
	irvWindow.FilterClose:SetSize(100,20)
	irvWindow.FilterClose:SetPosition((irvWindow.FilterPanel:GetWidth()-irvWindow.FilterClose:GetWidth())/2,irvWindow.FilterPanel:GetHeight()-25)
	irvWindow.FilterClose:SetText("Close")
	irvWindow.FilterClose.Click=function()
		irvWindow.FilterShow:SetVisible(true)
		irvWindow.FilterPanel:SetVisible(false)
	end

	irvWindow.Update = function(sender,args)
		if irvWindow.Loading and (Plugins["IRV"] ~= nil) then
			Plugins["IRV"].Unload = function(irvWindow,sender,args)
				UnloadPlugin();
			end
			irvWindow.Loading=false;
			Turbine.Shell.WriteLine("IRV "..Plugins["IRV"]:GetVersion().." by Garan loaded");
			irvWindow:SetWantsUpdates(false);
		end
	end       

	irvWindow.btnIO=Turbine.UI.Button();
	irvWindow.btnIO:SetParent(irvWindow)
	irvWindow.btnIO:SetSize(26,20);
	irvWindow.btnIO:SetPosition(irvWindow:GetWidth()-121,irvWindow.resourceCaption:GetTop());
	irvWindow.btnIO:SetBackground("GaranStuff/IRV/Resources/IOButton.jpg");
	irvWindow.btnIO.Click=function(sender,args)
		-- put IO stuff here
		ioWindow:SetVisible(true);
	end

	irvWindow.btnClose=Turbine.UI.Button();
	irvWindow.btnClose:SetParent(irvWindow)
	irvWindow.btnClose:SetSize(26,20);
	irvWindow.btnClose:SetPosition(irvWindow:GetWidth()-28,irvWindow.resourceCaption:GetTop());
	irvWindow.btnClose:SetBackground("GaranStuff/IRV/Resources/CloseButton.jpg");
	irvWindow.btnClose.Click=function(sender,args)
		irvWindow:SetVisible(false);
	end

	irvWindow.resourceCaption:SetZOrder(irvWindow.imageSolo:GetZOrder()+1)
	irvWindow.resourceID:SetZOrder(irvWindow.resourceCaption:GetZOrder()+1)
	irvWindow.resourceDescription:SetZOrder(irvWindow.resourceID:GetZOrder()+1)
	irvWindow.imageSize:SetZOrder(irvWindow.resourceDescription:GetZOrder()+1)
	irvWindow.btnPrevIDs:SetZOrder(irvWindow.imageSize:GetZOrder()+1)
	irvWindow.btnNextIDs:SetZOrder(irvWindow.btnPrevIDs:GetZOrder()+1)
	irvWindow.btnGridToggle:SetZOrder(irvWindow.btnNextIDs:GetZOrder()+1)
	irvWindow.btnFormatToggle:SetZOrder(irvWindow.btnGridToggle:GetZOrder()+1)
	irvWindow.TargetList:SetZOrder(irvWindow.btnGridToggle:GetZOrder()+1)
	irvWindow.TypeList:SetZOrder(irvWindow.TargetList:GetZOrder()+1)
	irvWindow.FilterHeight:SetZOrder(irvWindow.btnFormatToggle:GetZOrder())

	-- capture Key events
	irvWindow.KeyDown=function(sender, args)
--Turbine.Shell.WriteLine("KeyDown Action="..args.Action..", .Alt="..tostring(args.Alt)..", .Control="..tostring(args.Control)..", .Shift="..tostring(args.Shift))
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			sender:SetVisible( false );
		end
	end

	irvWindow.VisibleChanged=function()
		if (not irvWindow:IsVisible()) then
			-- hide all the other windows when we are hidden
			if libWindow~=nil then
				libWindow:SetVisible(false);
			end
			if editLibWindow~=nil then
				editLibWindow:SetVisible(false);
			end
			if catMaintWindow~=nil then
				catMaintWindow:SetVisible(false);
			end
			if ioWindow~=nil then
				ioWindow:SetVisible(false);
			end
		end
	end

	-- register shell commands
	irvWindow.shellCmd=Turbine.ShellCommand();
	local numberOfCommandsRegistered = Turbine.Shell.AddCommand("irv;ImageResourceViewer",irvWindow.shellCmd);
	irvWindow.shellCmd.Execute = function(sender, cmd, args)
		if (cmd == "irv" or cmd=="ImageResourceViewer") then
			if (args=="show") then
				irvWindow:SetVisible(true);
			elseif (args=="hide") then
				irvWindow:SetVisible(false);
			elseif (args=="toggle") then
				irvWindow:SetVisible(not irvWindow:IsVisible());
			elseif (args=="debug") then
				debugWindow:SetVisible(true)
			elseif (args=="adm") then
				irvWindow.TargetCaption:SetVisible(true)
				irvWindow.TargetList:SetVisible(true)
				irvWindow.imageMenu:GetItems():Get(5):SetEnabled(true)
				adm=true
			else
				Turbine.Shell.WriteLine("usage:");
				Turbine.Shell.WriteLine("  /irv show");
				Turbine.Shell.WriteLine("       Displays the Image Resource Viewer.");
				Turbine.Shell.WriteLine("  /irv hide");
				Turbine.Shell.WriteLine("       Hides the Image Resource Viewer.");
				Turbine.Shell.WriteLine("  /irv toggle");
				Turbine.Shell.WriteLine("       Toggles the Image Resource Viewer (shows it if it was hidden, hides it if it was showing).");
			end
		end
	end
	irvWindow.shellCmd.GetHelp = function(sender, cmd)
		return("usage:\n  /irv show\n       Displays the Image Resource Viewer.\n  /irv hide\n       Hides the Image Resource Viewer.\n  /irv toggle\n       Toggles the Image Resource Viewer (shows it if it was hidden, hides it if it was showing).");
	end

	irvWindow.shellCmd.GetShortHelp = function(sender, cmd)
		return("irv - usage: /irv show | hide | toggle");
	end
	irvWindow:SetWantsKeyEvents(true);

	irvWindow.imageMenu=Turbine.UI.ContextMenu();
	irvWindow.imageMenu:GetItems():Add(Turbine.UI.MenuItem("Category"))
	irvWindow.imageMenu:GetItems():Add(Turbine.UI.MenuItem("SubCategory"))
	irvWindow.imageMenu:GetItems():Add(Turbine.UI.MenuItem("Description"))
	irvWindow.imageMenu:GetItems():Get(1):SetEnabled(false);
	irvWindow.imageMenu:GetItems():Get(2):SetEnabled(false);
	irvWindow.imageMenu:GetItems():Get(3):SetEnabled(false);
	irvWindow.imageMenu:GetItems():Add(Turbine.UI.MenuItem("Edit Library Entry"));
-- disable the library entry maintenance until we figure out what to do with the category/subcategory structures, etc.
irvWindow.imageMenu:GetItems():Get(4):SetEnabled(false)
	irvWindow.imageMenu:GetItems():Add(Turbine.UI.MenuItem("Transfer Group"));
	irvWindow.imageMenu:GetItems():Get(5):SetEnabled(adm)
	irvWindow.imageMenu:GetItems():Get(5).Click=function()

		if irvWindow.imageGrid.clickRow~=nil then
			irvWindow.imageGrid[irvWindow.imageGrid.clickRow][irvWindow.imageGrid.clickCol].selected=true
		end
		local sourceType=irvWindow.TypeList:GetValue()
		local targetType=irvWindow.TargetList:GetValue()
		if sourceType~=targetType then
			local sourceLib,sourceIndex=GetLibraryByType(sourceType)
			local targetLib,targetIndex=GetLibraryByType(targetType)
			local nextResource=tonumber(irvWindow.resourceID:GetText())
			if irvWindow.ShowGrid then
				nextResource=GetNextResource(irvWindow.imageGrid[irvWindow.GridRows][irvWindow.GridCols].ResourceID)
				local found=false
				for tmpRow=1,irvWindow.GridRows do
					for tmpCol=1,irvWindow.GridCols do
						local tmpCell=irvWindow.imageGrid[tmpRow][tmpCol]
						if tmpCell.selected then
							targetLib[tmpCell.ResourceID]={}
							local targetResource=targetLib[tmpCell.ResourceID]
							for k,v in pairs(sourceLib[tmpCell.ResourceID]) do
								targetResource[k]=v
							end
							sourceLib[tmpCell.ResourceID]=nil
						elseif not found and irvWindow.imageGrid[tmpRow][tmpCol]:IsVisible() then
							nextResource=irvWindow.imageGrid[tmpRow][tmpCol].ResourceID
							found=true
						end
					end
				end
			else
				local resource=tonumber(irvWindow.resourceID:GetText())
				nextResource=GetNextResource(resource)
				targetLib[resource]={}
				local targetResource=targetLib[resource]
				for k,v in pairs(sourceLib[resource]) do
					targetResource[k]=v
				end
				sourceLib[resource]=nil
			end
			RebuildIndex(sourceLib)
			RebuildIndex(targetLib)
			irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,nextResource))
		end
		ShowBackgrounds(false)
	end
	irvWindow.DisableControls=function()
		irvWindow.resourceID:SetEnabled(false);
		irvWindow.btnPrevIDs:SetEnabled(false);
		irvWindow.btnGoToImage:SetEnabled(false);
		irvWindow.btnNextIDs:SetEnabled(false);
		irvWindow.btnGridToggle:SetEnabled(false);
		-- we let the format toggle remain enabled since it's harmless
		irvWindow.btnSearch:SetEnabled(false);
		irvWindow.btnAutoPrev:SetEnabled(false);
		irvWindow.btnAutoNext:SetEnabled(false);
		irvWindow.btnIO:SetEnabled(false);

		irvWindow.imageSolo:SetMouseVisible(false);
	end
	irvWindow.EnableControls=function()
		irvWindow.resourceID:SetEnabled(true);
		irvWindow.btnPrevIDs:SetEnabled(true);
		irvWindow.btnGoToImage:SetEnabled(true);
		irvWindow.btnNextIDs:SetEnabled(true);
		irvWindow.btnGridToggle:SetEnabled(true);
		-- we let the format toggle remain enabled since it's harmless
		irvWindow.btnSearch:SetEnabled(true);
		irvWindow.btnAutoPrev:SetEnabled(true);
		irvWindow.btnAutoNext:SetEnabled(true);
		irvWindow.btnIO:SetEnabled(true);

		irvWindow.imageSolo:SetMouseVisible(true);
	end
function GetNextResource(tmpResource)
	local selectedType=irvWindow.TypeList:GetValue()
	if selectedType==1 then
		tmpIndex=ResourceIcons
		tmpLibrary=LibraryIcons
	elseif selectedType==2 then
		tmpIndex=ResourceMaps
		tmpLibrary=LibraryMaps
	elseif selectedType==3 then
		tmpIndex=ResourceScenes
		tmpLibrary=LibraryScenes
	elseif selectedType==4 then
		tmpIndex=ResourceTexture
		tmpLibrary=LibraryTexture
	elseif selectedType==5 then
		tmpIndex=ResourceOther
		tmpLibrary=LibraryOther
	elseif selectedType==6 then
		tmpIndex=ResourceTiles
		tmpLibrary=LibraryTiles
	else
		tmpIndex=ResourceTerrain
		tmpLibrary=LibraryTerrain
	end
	local maxIndex=#tmpIndex
	if tmpIndex[1]~=nil then
		-- find the next resource with this type
		if tmpLibrary[tmpResource]==nil then
			local min=1
			local max=maxIndex
			local cur=math.floor((max-min)/2)
			if tmpResource<tmpIndex[min] then
				tmpResource=0
				cur=min
			end
			if tmpResource>tmpIndex[max] then
				tmpResource=tmpIndex[max]
				cur=max
				min=max
			end
			while tmpIndex[cur]~=tmpResource and min~=max and cur~=min do
				if tmpIndex[cur]<tmpResource then
					min=cur
					cur=math.floor(min+(max-min)/2)
				elseif tmpIndex[cur]>tmpResource then
					max=cur
					cur=math.floor(min+(max-min)/2)
				end
			end
			if tmpResource>0 and tmpIndex[cur]~=tmpResource then tmpResource=tmpIndex[cur] end
		end
		-- now get next
		if tmpResource==0 then
			tmpResource=tmpIndex[1]
		else
			local tmpEntry=tmpLibrary[tmpResource]
			if tmpEntry~=nil then
				local newIndex=tmpEntry[6]+1
				if newIndex>maxIndex then newIndex=maxIndex end
				tmpResource=tmpIndex[newIndex]
			end
		end
		filterWidth=tonumber(irvWindow.FilterWidth:GetText())
		if filterWidth==nil then filterWidth=0 end
		filterHeight=tonumber(irvWindow.FilterHeight:GetText())
		if filterHeight==nil then filterHeight=0 end
		if filterWidth~=0 or filterHeight~=0 then
			while tmpLibrary[tmpResource]~=nil and tmpLibrary[tmpResource][6]<maxIndex and ((filterWidth~=0 and tmpLibrary[tmpResource][2]~=filterWidth) or (filterHeight~=0 and tmpLibrary[tmpResource][3]~=filterHeight)) do
				tmpResource=tmpIndex[tmpLibrary[tmpResource][6]+1]
			end
		end
	end
	return tmpResource
end

function GetPrevResource(tmpResource)
	local selectedType=irvWindow.TypeList:GetValue()
	if selectedType==1 then
		tmpIndex=ResourceIcons
		tmpLibrary=LibraryIcons
	elseif selectedType==2 then
		tmpIndex=ResourceMaps
		tmpLibrary=LibraryMaps
	elseif selectedType==3 then
		tmpIndex=ResourceScenes
		tmpLibrary=LibraryScenes
	elseif selectedType==4 then
		tmpIndex=ResourceTexture
		tmpLibrary=LibraryTexture
	elseif selectedType==5 then
		tmpIndex=ResourceOther
		tmpLibrary=LibraryOther
	elseif selectedType==6 then
		tmpIndex=ResourceTiles
		tmpLibrary=LibraryTiles
	else
		tmpIndex=ResourceTerrain
		tmpLibrary=LibraryTerrain
	end
	local maxIndex=#tmpIndex

	if tmpIndex[1]~=nil then
		if tmpLibrary[tmpResource]==nil then
			local min=1
			local max=maxIndex
			local cur=math.floor((max-min)/2)
			if tmpResource<tmpIndex[min] then
				tmpResource=tmpIndex[min]
				cur=min
			end
			if tmpResource>tmpIndex[max] then
				tmpResource=0
				cur=max
				min=max
			end
			while tmpIndex[cur]~=tmpResource and min~=max and cur~=min do
				if tmpIndex[cur]<tmpResource then
					min=cur
					cur=math.floor(min+(max-min)/2)
				elseif tmpIndex[cur]>tmpResource then
					max=cur
					cur=math.floor(min+(max-min)/2)
				end
			end
			if tmpResource~=0 and tmpIndex[cur]~=tmpResource then
				tmpResource=tmpIndex[cur+1]
			end
		end
		-- now get prev
		if tmpResource==0 then
			tmpResource=tmpIndex[maxIndex]
		else
			local tmpEntry=tmpLibrary[tmpResource]
			if tmpEntry~=nil then
				local newIndex=tmpEntry[6]-1
				if newIndex<1 then newIndex=1 end
				tmpResource=tmpIndex[newIndex]
			end
		end
		filterWidth=tonumber(irvWindow.FilterWidth:GetText())
		if filterWidth==nil then filterWidth=0 end
		filterHeight=tonumber(irvWindow.FilterHeight:GetText())
		if filterHeight==nil then filterHeight=0 end
		if filterWidth~=0 or filterHeight~=0 then
			while tmpResource~=nil and tmpLibrary[tmpResource]~=nil and tmpLibrary[tmpResource][6]<maxIndex and ((filterWidth~=0 and tmpLibrary[tmpResource][2]~=filterWidth) or (filterHeight~=0 and tmpLibrary[tmpResource][3]~=filterHeight)) do
				tmpResource=tmpIndex[tmpLibrary[tmpResource][6]-1]
			end
		end
	end
	return tmpResource
end

function ShowBackgrounds(reverse)
	inShowBackgrounds=true
	local resource=tonumber(irvWindow.resourceID:GetText());
	-- for reverse, we simply move back x number of times then show forward. that prevents the need to check for "first" image
	if reverse then
		if irvWindow.ShowGrid then
			for i=1,#irvWindow.imageGrid*#irvWindow.imageGrid[1]-1 do
				local tmpResource=GetPrevResource(resource)
				if tmpResource~=nil then
					resource=tmpResource
				end
			end
		end
		irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,resource))
	end
	local a,b,c,d=GetLibraryType(resource)
	if c~=nil and c[resource]~=nil and d~=nil then
		local tmpMax=#d
		irvWindow.XofY:SetText(tostring(c[resource][6]).."/"..tostring(tmpMax))
		irvWindow.XofYScroll:SetMaximum(tmpMax)
		irvWindow.XofYScroll:SetValue(c[resource][6]) -- might be an issue if it toggles another ShowBackgrounds
	else
		irvWindow.XofY:SetText("")
	end
	irvWindow.imageGrid.selectStartRow=nil
	irvWindow.imageGrid.selectStartCol=nil
	if irvWindow.ShowGrid==true then
		local iRow,iCol,tmpRow,tmpCol
		local imageWidth,imageHeight;
		local cellMaxWidth=irvWindow.imageGrid[1][1]:GetWidth()-12;
		local cellMaxHeight=irvWindow.imageGrid[1][1]:GetHeight()-30;
		irvWindow.imageSolo:SetVisible(false);
		irvWindow.imageSize:SetVisible(false);
		irvWindow.resourceDescription:SetText("");
		local oldResource=0
		for iRow=1, #irvWindow.imageGrid do
			for iCol=1,#irvWindow.imageGrid[iRow] do
				irvWindow.imageGrid[iRow][iCol].selected=false
				irvWindow.imageGrid[iRow][iCol]:SetBackColor(Turbine.UI.Color(.1,.1,.1));
				irvWindow.imageGrid[iRow][iCol]:SetVisible(oldResource~=resource);
				irvWindow.imageGrid[iRow][iCol].imageID:SetText(string.format(irvWindow.numberFormat,resource));
				irvWindow.imageGrid[iRow][iCol].ResourceID=resource;
				irvWindow.imageGrid[iRow][iCol].image:SetVisible(true);
				irvWindow.imageGrid[iRow][iCol].image:SetSize(1,1);
-- pcall
				local success,result=pcall(Turbine.UI.Control.SetBackground,irvWindow.imageGrid[iRow][iCol].image,resource);
				irvWindow.imageGrid[iRow][iCol].image:SetStretchMode(2); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
				imageWidth,imageHeight=irvWindow.imageGrid[iRow][iCol].image:GetSize();
				irvWindow.imageGrid[iRow][iCol].imageX=imageWidth;
				irvWindow.imageGrid[iRow][iCol].imageY=imageHeight;
				if iRow>1 or iCol>1 then
					tmpWidth=tonumber(irvWindow.FilterWidth:GetText())
					tmpHeight=tonumber(irvWindow.FilterHeight:GetText())
					if (tmpWidth~=nil and tmpWidth~=imageWidth) or (tmpHeight~=nil and tmpHeight~=imageHeight) then
						irvWindow.imageGrid[iRow][iCol]:SetVisible(false)
					end
				end
				irvWindow.imageGrid[iRow][iCol].imageSize:SetText(imageWidth.." x "..imageHeight);
				if irvWindow.StretchThumbs:IsChecked() and (imageWidth>cellMaxWidth or imageHeight>cellMaxHeight) then
					-- determine which axis has greater ratio
					irvWindow.imageGrid[iRow][iCol].image:SetStretchMode(1);
					irvWindow.imageGrid[iRow][iCol].image:SetBlendMode(2)
					if imageWidth/cellMaxWidth>imageHeight/cellMaxHeight then
						imageHeight=math.floor(imageHeight*cellMaxWidth/imageWidth)
						imageWidth=cellMaxWidth
					else
						imageWidth=math.floor(imageWidth*cellMaxHeight/imageHeight)
						imageHeight=cellMaxHeight
					end
				else
					irvWindow.imageGrid[iRow][iCol].image:SetStretchMode(0);
					irvWindow.imageGrid[iRow][iCol].image:SetBlendMode(5)
					if imageWidth>cellMaxWidth then imageWidth=cellMaxWidth end;
					if imageHeight>cellMaxHeight then imageHeight=cellMaxHeight end;
				end
				irvWindow.imageGrid[iRow][iCol].image:SetSize(imageWidth,imageHeight);
				irvWindow.imageGrid[iRow][iCol].image:SetPosition((cellMaxWidth-imageWidth)/2+1,(cellMaxHeight-imageHeight)/2+1);
				oldResource=resource
				resource=GetNextResource(resource)
			end
		end
	else
		for iRow=1, #irvWindow.imageGrid do
			for iCol=1,#irvWindow.imageGrid[iRow] do
				irvWindow.imageGrid[iRow][iCol]:SetVisible(false);
				irvWindow.imageGrid[iRow][iCol].selected=false
				irvWindow.imageGrid[iRow][iCol]:SetBackColor(Turbine.UI.Color(.1,.1,.1));
			end
		end
		irvWindow.imageSize:SetVisible(true);
		irvWindow.resourceID:SetText(string.format(irvWindow.numberFormat,resource)); -- refresh the number using the current format
		irvWindow.imageSolo:SetVisible(true);
		irvWindow.imageSolo:SetSize(100,100);
		local success,result=pcall(Turbine.UI.Control.SetBackground,irvWindow.imageSolo,resource);
		irvWindow.imageSolo:SetStretchMode(2); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
		irvWindow.imageWidth,irvWindow.imageHeight=irvWindow.imageSolo:GetSize();
		irvWindow.imageSize:SetText(irvWindow.imageWidth.." x "..irvWindow.imageHeight);
		irvWindow.imageSolo:SetStretchMode(0); -- setting stretchmode(2) AFTER loading an image will resize the control to the size of the image
		irvWindow.imageSolo:SetSize(irvWindow.imageWidth,irvWindow.imageHeight); -- for some reason, we have to manually set the image size even though the width and height are reported as having changed already
		irvWindow.imageSolo:SetPosition((irvWindow:GetWidth()-irvWindow.imageWidth)/2,(irvWindow:GetHeight()-irvWindow.imageHeight)/2)
		local cat,subcat,desc=GetLibraryInfo(resource);
		if cat~="" then
			if subcat==nil or subcat=="" then
				desc=tostring(cat).." - "..tostring(desc);
			else
				desc=tostring(cat).." / "..tostring(subcat).." - "..tostring(desc);
			end
		end
		irvWindow.resourceDescription:SetText(desc);
	end
	inShowBackgrounds=false
end

if #Categories==0 or #Library==0 then
	import "GaranStuff.IRV.Defaults";
	LoadDefaults()
end

-- Instantiate the actual windows, etc

libWindow=LibraryWindow();
libWindow:SetVisible(false);
editLibWindow=EditLibraryWindow();
editLibWindow:SetVisible(false);
catMaintWindow=CategoryMaintWindow();
catMaintWindow:SetVisible(false);
ioWindow:SetVisible(false);

irvWindow:SetZOrder(irvWindow:GetZOrder()+1); -- force irv to "Topmost"
libWindow:SetZOrder(irvWindow:GetZOrder()+1);
editLibWindow:SetZOrder(libWindow:GetZOrder()+1);
catMaintWindow:SetZOrder(editLibWindow:GetZOrder()+1);
ioWindow:SetZOrder(catMaintWindow:GetZOrder()+1);

irvWindow:SetVisible(true);

ShowBackgrounds(false);

debugWindow=DebugWindow()